@@ -11,7 +11,7 @@ static char seed[128];
 char ip[4];
 static stralloc out;
 
-main(int argc,char **argv)
+int main(int argc,char **argv)
 {
   dns_random_init(seed);
 
