@@ -7,27 +7,32 @@
 #include "strerr.h"
 #include "uint16.h"
 #include "response.h"
+#include "case.h"
 #include "printpacket.h"
 #include "parsetype.h"
+#include "ip4.h"
 #include "dns.h"
 
+extern int respond(char *,char *,char *);
+
 #define FATAL "tinydns-get: fatal: "
 
 void usage(void)
 {
-  strerr_die1x(100,"tinydns-get: usage: tinydns-get type name");
+  strerr_die1x(100,"tinydns-get: usage: tinydns-get type name [ip]");
 }
 void oops(void)
 {
   strerr_die2sys(111,FATAL,"unable to parse: ");
 }
 
-char type[2];
+static char ip[4];
+static char type[2];
 static char *q;
 
 static stralloc out;
 
-main(int argc,char **argv)
+int main(int argc,char **argv)
 {
   uint16 u16;
 
@@ -39,6 +44,10 @@ main(int argc,char **argv)
   if (!*++argv) usage();
   if (!dns_domain_fromdot(&q,*argv,str_len(*argv))) oops();
 
+  if (*++argv) {
+    if (!ip4_scan(*argv,ip)) usage();
+  }
+
   if (!stralloc_copys(&out,"")) oops();
   uint16_unpack_big(type,&u16);
   if (!stralloc_catulong0(&out,u16,0)) oops();
@@ -46,7 +55,7 @@ main(int argc,char **argv)
   if (!dns_domain_todot_cat(&out,q)) oops();
   if (!stralloc_cats(&out,":\n")) oops();
 
-  if (!response_query(q,type)) oops();
+  if (!response_query(q,type,DNS_C_IN)) oops();
   response[3] &= ~128;
   response[2] &= ~1;
   response[2] |= 4;
@@ -57,7 +66,7 @@ main(int argc,char **argv)
     response[3] |= 4;
   }
   else
-    if (!respond(q,type,"\0\0\0\0")) goto DONE;
+    if (!respond(q,type,ip)) goto DONE;
 
   if (!printpacket_cat(&out,response,response_len)) oops();
 
